//---------------------------------------------------------------------
// ImageFilterTest.cs (c) 2007 by Charles Petzold for "Beautiful Code"
//---------------------------------------------------------------------
using System;
using System.Drawing;
using System.Windows.Forms;

class ImageFilterTest : ImageClip
{
    MenuItem miScale;
    MenuItem miGenerate;

    [STAThread]
    public new static void Main()
    {
        Application.Run(new ImageFilterTest());
    }

    public ImageFilterTest()
    {
        Text = strProgName = "Test filtru obrazw";

        // Menu View
        MenuItem miView = new MenuItem("&View");
        Menu.MenuItems.Add(miView);

        // Element skalowania
        miScale = new MenuItem("&Skalowanie izotropowe do okna");
        miScale.Click += new EventHandler(MenuViewIsotropicallyOnClick);
        miScale.Checked = true;
        miView.MenuItems.Add(miScale);

        // menu Filter
        MenuItem miFilter = new MenuItem("F&ilter");
        Menu.MenuItems.Add(miFilter);

        // Element Generate-Code
        miGenerate = new MenuItem("&Uyj metody, ktra generuje jzyk poredni");
        miGenerate.Click += new EventHandler(MenuFilterGenerateOnClick);
        miFilter.MenuItems.Add(miGenerate);

        // Elementy filtra
        MenuItem mi = new MenuItem("3 \x00D7 3 Filtr rozmazujcy");
        mi.Click += new EventHandler(FilterOnClick);
        mi.Tag = new ImageFilter(3, new double[] {1, 1, 1, 1, 1, 1, 1, 1, 1});
        miFilter.MenuItems.Add(mi);

        mi = new MenuItem("5 \x00D7 5 Filtr rozmazujcy");
        mi.Click += new EventHandler(FilterOnClick);
        mi.Tag = new ImageFilter(5, new double[] {1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                                                  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
                                                  1, 1, 1, 1, 1 });
        miFilter.MenuItems.Add(mi);

        mi = new MenuItem("3 \x00D7 3 Filtr wyostrzajcy (0 -1 0, -1 5 -1, 0 -1 0)");
        mi.Click += new EventHandler(FilterOnClick);
        mi.Tag = new ImageFilter(3, new double[] { 0, -1, 0, -1, 5, -1, 0, -1, 0 });
        miFilter.MenuItems.Add(mi);

        mi = new MenuItem("3 \x00D7 3 Filtr wyostrzajcy (-1 -1 -1, -1 9 -1, -1 -1 -1)");
        mi.Click += new EventHandler(FilterOnClick);
        mi.Tag = new ImageFilter(3, new double[] {-1, -1, -1, -1, 9, -1, -1, -1, -1});
        miFilter.MenuItems.Add(mi);

        mi = new MenuItem("3 \x00D7 3 Filtr wykrywania krawdzi (-1 -1 -1, -1 8 -1, -1 -1 -1)");
        mi.Click += new EventHandler(FilterOnClick);
        mi.Tag = new ImageFilter(3, new double[] { -1, -1, -1, -1, 8, -1, -1, -1, -1 });
        miFilter.MenuItems.Add(mi);
    }

    // Procedura obsugi OnPaint wywietlajca peny rozmiar mapy bitowej lub rozmiar okna.
    protected override void OnPaint(PaintEventArgs args)
    {
        if (miScale.Checked)
            base.OnPaint(args);

        else if (image != null)
        {
            Graphics grfx = args.Graphics;
            grfx.DrawImage(image, 0, 0);
        }
    }

    // Dwie procedury obsugi elementw menu, ktre mona zaznacza.
    void MenuViewIsotropicallyOnClick(object sender, EventArgs args)
    {
        miScale.Checked = !miScale.Checked;
        Invalidate();
    }
    void MenuFilterGenerateOnClick(object sender, EventArgs args)
    {
        miGenerate.Checked = !miGenerate.Checked;
    }

    // Procedura obsugi zdarze menu
    void FilterOnClick(object sender, EventArgs args)
    {
        if (!(image is Bitmap))
        {
            MessageBox.Show("Filtr moe by stosowany tylko do kolorowych map bitowych!", strProgName);
            return;
        }

        MenuItem mi = sender as MenuItem;

        Bitmap bmp = image as Bitmap;
        ImageFilter filter = mi.Tag as ImageFilter;

        Cursor cursorSave = Cursor.Current;
        Cursor.Current = Cursors.WaitCursor;

        DateTime dt = DateTime.Now;
        bool isSuccess = filter.ApplyFilter(bmp, miGenerate.Checked);
        TimeSpan ts = DateTime.Now - dt;

        Cursor.Current = cursorSave;

        if (isSuccess)
        {
            Invalidate();
            MessageBox.Show("Upyno czasu = " + ts.TotalMilliseconds + " milisekund", strProgName);
        }
        else
        {
            MessageBox.Show("Mapa bitowa musi zawiera 24 lub 32 bity w pikselu!", strProgName);
        }
    }
}
